package se.chalmers.tdv.hashtable;

import java.util.Random;

import se.chalmers.tdv.testing.StatefulTestCase;

/**
 * Test suite for the hash table.
 */
public class LLHashTableTestSuite {

	/**
	 * Testing with empty commands, effectively checking that the same prefix
	 * and suffix leave the hash tables in the same state.
	 */
	public static final LLHashTableTest testRandom = new LLHashTableTest() {

		public String name() {
			return "random";
		}

		public void initialize(
				StatefulTestCase<LLHashTable, LLHashTableAction, Integer> testCase) {
			testCase.setCommands(new LLHashTableAction[] {},
					new LLHashTableAction[] {});
		}
	};

	/**
	 * Testing that put(Ka,Va), put(Kb,Vb) <==> put(Kb,Vb), put(Ka, Va) iff Ka
	 * != Kb.
	 */
	public static final LLHashTableTest commutePut = new LLHashTableTest() {

		public void initialize(
				StatefulTestCase<LLHashTable, LLHashTableAction, Integer> testCase) {
			Random rand = new Random();
			int keyA = rand.nextInt(LLHashTableTestCase.MAX_KEY);
			int keyB;
			do {
				keyB = rand.nextInt(LLHashTableTestCase.MAX_KEY);
			} while (keyB == keyA);

			int valueA = rand.nextInt();
			int valueB = rand.nextInt();
			LLHashTableAction A = LLHashTableAction.newPutAction(keyA, valueA);
			LLHashTableAction B = LLHashTableAction.newPutAction(keyB, valueB);
			testCase.setCommands(new LLHashTableAction[] { A, B },
					new LLHashTableAction[] { B, A });
		}

		public String name() {
			return "commutePut";
		}

	};

	/** Runs until all test are successful or a bug is found. */
	public static void runAllTests() {
		int NTEST = 1000;

		LLHashTableTest[] tests = { testRandom, commutePut };

		for (LLHashTableTest test : tests) {
			System.out.print("Testing " + test.name() + "... ");
			int i = 1;
			for (; i <= NTEST; i++) {
				LLHashTableTestCase testCase = new LLHashTableTestCase(test);
				if (!testCase.initializeAndRunTest()) {
					System.out.println("\nBug found after " + i + " tests:");

					testCase.minimizeCapacity();
					testCase.minimizeContext();

					testCase.print();
					// Enable this to print out the code of this test case.
					// testCase.printTestCode();
					System.out.println();
					break;
				}
			}
			if (i > NTEST)
				System.out.println("OK " + NTEST + " tests.");
		}

	}

	public static void main(String[] arg) {
		runAllTests();
	}
}
